@@ -87,9 +87,9 @@ int faiss_index_cpu_to_gpu_multiple(
 }
 
 int faiss_index_cpu_to_gpu_multiple_with_options(
-        FaissGpuResourcesProvider** providers_vec,
+        FaissGpuResourcesProvider* const* providers_vec,
         size_t providers_vec_size,
-        int* devices,
+        const int* devices,
         size_t devices_size,
         const FaissIndex* index,
         const FaissGpuMultipleClonerOptions* options,
