@@ -135,7 +135,7 @@ int dgesvd_(
  * VectorTransform
  *********************************************/
 
-float* VectorTransform::apply(Index::idx_t n, const float* x) const {
+float* VectorTransform::apply(idx_t n, const float* x) const {
     float* xt = new float[n * d_out];
     apply_noalloc(n, x, xt);
     return xt;
@@ -166,8 +166,7 @@ LinearTransform::LinearTransform(int d_in, int d_out, bool have_bias)
     is_trained = false; // will be trained when A and b are initialized
 }
 
-void LinearTransform::apply_noalloc(Index::idx_t n, const float* x, float* xt)
-        const {
+void LinearTransform::apply_noalloc(idx_t n, const float* x, float* xt) const {
     FAISS_THROW_IF_NOT_MSG(is_trained, "Transformation not trained yet");
 
     float c_factor;
@@ -348,7 +347,7 @@ void RandomRotationMatrix::init(int seed) {
     is_trained = true;
 }
 
-void RandomRotationMatrix::train(Index::idx_t /*n*/, const float* /*x*/) {
+void RandomRotationMatrix::train(idx_t /*n*/, const float* /*x*/) {
     // initialize with some arbitrary seed
     init(12345);
 }
@@ -442,7 +441,7 @@ void eig(size_t d_in, double* cov, double* eigenvalues, int verbose) {
 
 } // namespace
 
-void PCAMatrix::train(Index::idx_t n, const float* x) {
+void PCAMatrix::train(idx_t n, const float* x) {
     const float* x_in = x;
 
     x = fvecs_maybe_subsample(
@@ -733,7 +732,7 @@ ITQMatrix::ITQMatrix(int d)
         : LinearTransform(d, d, false), max_iter(50), seed(123) {}
 
 /** translated from fbcode/deeplearning/catalyzer/catalyzer/quantizers.py */
-void ITQMatrix::train(Index::idx_t n, const float* xf) {
+void ITQMatrix::train(idx_t n, const float* xf) {
     size_t d = d_in;
     std::vector<double> rotation(d * d);
 
@@ -957,8 +956,7 @@ void ITQTransform::train(idx_t n, const float* x) {
     is_trained = true;
 }
 
-void ITQTransform::apply_noalloc(Index::idx_t n, const float* x, float* xt)
-        const {
+void ITQTransform::apply_noalloc(idx_t n, const float* x, float* xt) const {
     FAISS_THROW_IF_NOT_MSG(is_trained, "Transformation not trained yet");
 
     std::unique_ptr<float[]> x_norm(new float[n * d_in]);
@@ -1003,7 +1001,7 @@ OPQMatrix::OPQMatrix(int d, int M, int d2)
     pq = nullptr;
 }
 
-void OPQMatrix::train(Index::idx_t n, const float* x) {
+void OPQMatrix::train(idx_t n, const float* x) {
     const float* x_in = x;
 
     x = fvecs_maybe_subsample(d_in, (size_t*)&n, max_train_points, x, verbose);
@@ -1261,7 +1259,7 @@ CenteringTransform::CenteringTransform(int d) : VectorTransform(d, d) {
     is_trained = false;
 }
 
-void CenteringTransform::train(Index::idx_t n, const float* x) {
+void CenteringTransform::train(idx_t n, const float* x) {
     FAISS_THROW_IF_NOT_MSG(n > 0, "need at least one training vector");
     mean.resize(d_in, 0);
     for (idx_t i = 0; i < n; i++) {
