@@ -82,33 +82,32 @@ class GpuIndexFlat : public GpuIndex {
     void reset() override;
 
     /// This index is not trained, so this does nothing
-    void train(Index::idx_t n, const float* x) override;
+    void train(idx_t n, const float* x) override;
 
     /// Overrides to avoid excessive copies
-    void add(Index::idx_t, const float* x) override;
+    void add(idx_t, const float* x) override;
 
     /// Reconstruction methods; prefer the batch reconstruct as it will
     /// be more efficient
-    void reconstruct(Index::idx_t key, float* out) const override;
+    void reconstruct(idx_t key, float* out) const override;
 
     /// Batch reconstruction method
-    void reconstruct_n(Index::idx_t i0, Index::idx_t num, float* out)
-            const override;
+    void reconstruct_n(idx_t i0, idx_t num, float* out) const override;
 
     /// Batch reconstruction method
-    void reconstruct_batch(Index::idx_t n, const Index::idx_t* keys, float* out)
+    void reconstruct_batch(idx_t n, const idx_t* keys, float* out)
             const override;
 
     /// Compute residual
-    void compute_residual(const float* x, float* residual, Index::idx_t key)
+    void compute_residual(const float* x, float* residual, idx_t key)
             const override;
 
     /// Compute residual (batch mode)
     void compute_residual_n(
-            Index::idx_t n,
+            idx_t n,
             const float* xs,
             float* residuals,
-            const Index::idx_t* keys) const override;
+            const idx_t* keys) const override;
 
     /// For internal access
     inline FlatIndex* getGpuData() {
@@ -121,15 +120,15 @@ class GpuIndexFlat : public GpuIndex {
     bool addImplRequiresIDs_() const override;
 
     /// Called from GpuIndex for add
-    void addImpl_(int n, const float* x, const Index::idx_t* ids) override;
+    void addImpl_(idx_t n, const float* x, const idx_t* ids) override;
 
     /// Called from GpuIndex for search
     void searchImpl_(
-            int n,
+            idx_t n,
             const float* x,
             int k,
             float* distances,
-            Index::idx_t* labels,
+            idx_t* labels,
             const SearchParameters* params) const override;
 
    protected:
