@@ -34,7 +34,7 @@ struct ScalarQuantizer : Quantizer {
         QT_6bit,        ///< 6 bits per component
     };
 
-    QuantizerType qtype;
+    QuantizerType qtype = QT_8bit;
 
     /** The uniform encoder can estimate the range of representable
      * values of the unform encoder using different statistics. Here
@@ -48,11 +48,11 @@ struct ScalarQuantizer : Quantizer {
         RS_optim,     ///< alternate optimization of reconstruction error
     };
 
-    RangeStat rangestat;
-    float rangestat_arg;
+    RangeStat rangestat = RS_minmax;
+    float rangestat_arg = 0;
 
     /// bits per scalar code
-    size_t bits;
+    size_t bits = 0;
 
     /// trained values (including the range)
     std::vector<float> trained;
