@@ -10,23 +10,32 @@
 #include <faiss/invlists/InvertedLists.h>
 
 #include <cstdio>
+#include <memory>
 
 #include <faiss/impl/FaissAssert.h>
 #include <faiss/utils/utils.h>
 
 namespace faiss {
 
+InvertedListsIterator::~InvertedListsIterator() {}
+
 /*****************************************
  * InvertedLists implementation
  ******************************************/
 
 InvertedLists::InvertedLists(size_t nlist, size_t code_size)
-        : nlist(nlist), code_size(code_size) {}
+        : nlist(nlist), code_size(code_size), use_iterator(false) {}
 
 InvertedLists::~InvertedLists() {}
 
-InvertedLists::idx_t InvertedLists::get_single_id(size_t list_no, size_t offset)
-        const {
+bool InvertedLists::is_empty(size_t list_no) const {
+    return use_iterator
+            ? !std::unique_ptr<InvertedListsIterator>(get_iterator(list_no))
+                       ->is_available()
+            : list_size(list_no) == 0;
+}
+
+idx_t InvertedLists::get_single_id(size_t list_no, size_t offset) const {
     assert(offset < list_size(list_no));
     const idx_t* ids = get_ids(list_no);
     idx_t id = ids[offset];
@@ -67,6 +76,10 @@ void InvertedLists::reset() {
     }
 }
 
+InvertedListsIterator* InvertedLists::get_iterator(size_t /*list_no*/) const {
+    FAISS_THROW_MSG("get_iterator is not supported");
+}
+
 void InvertedLists::merge_from(InvertedLists* oivf, size_t add_id) {
 #pragma omp parallel for
     for (idx_t i = 0; i < nlist; i++) {
@@ -87,6 +100,98 @@ void InvertedLists::merge_from(InvertedLists* oivf, size_t add_id) {
     }
 }
 
+size_t InvertedLists::copy_subset_to(
+        InvertedLists& oivf,
+        subset_type_t subset_type,
+        idx_t a1,
+        idx_t a2) const {
+    FAISS_THROW_IF_NOT(nlist == oivf.nlist);
+    FAISS_THROW_IF_NOT(code_size == oivf.code_size);
+    FAISS_THROW_IF_NOT_FMT(
+            subset_type >= 0 && subset_type <= 4,
+            "subset type %d not implemented",
+            subset_type);
+    size_t accu_n = 0;
+    size_t accu_a1 = 0;
+    size_t accu_a2 = 0;
+    size_t n_added = 0;
+
+    size_t ntotal = 0;
+    if (subset_type == 2) {
+        ntotal = compute_ntotal();
+    }
+
+    for (idx_t list_no = 0; list_no < nlist; list_no++) {
+        size_t n = list_size(list_no);
+        ScopedIds ids_in(this, list_no);
+
+        if (subset_type == SUBSET_TYPE_ID_RANGE) {
+            for (idx_t i = 0; i < n; i++) {
+                idx_t id = ids_in[i];
+                if (a1 <= id && id < a2) {
+                    oivf.add_entry(
+                            list_no,
+                            get_single_id(list_no, i),
+                            ScopedCodes(this, list_no, i).get());
+                    n_added++;
+                }
+            }
+        } else if (subset_type == SUBSET_TYPE_ID_MOD) {
+            for (idx_t i = 0; i < n; i++) {
+                idx_t id = ids_in[i];
+                if (id % a1 == a2) {
+                    oivf.add_entry(
+                            list_no,
+                            get_single_id(list_no, i),
+                            ScopedCodes(this, list_no, i).get());
+                    n_added++;
+                }
+            }
+        } else if (subset_type == SUBSET_TYPE_ELEMENT_RANGE) {
+            // see what is allocated to a1 and to a2
+            size_t next_accu_n = accu_n + n;
+            size_t next_accu_a1 = next_accu_n * a1 / ntotal;
+            size_t i1 = next_accu_a1 - accu_a1;
+            size_t next_accu_a2 = next_accu_n * a2 / ntotal;
+            size_t i2 = next_accu_a2 - accu_a2;
+
+            for (idx_t i = i1; i < i2; i++) {
+                oivf.add_entry(
+                        list_no,
+                        get_single_id(list_no, i),
+                        ScopedCodes(this, list_no, i).get());
+            }
+
+            n_added += i2 - i1;
+            accu_a1 = next_accu_a1;
+            accu_a2 = next_accu_a2;
+        } else if (subset_type == SUBSET_TYPE_INVLIST_FRACTION) {
+            size_t i1 = n * a2 / a1;
+            size_t i2 = n * (a2 + 1) / a1;
+
+            for (idx_t i = i1; i < i2; i++) {
+                oivf.add_entry(
+                        list_no,
+                        get_single_id(list_no, i),
+                        ScopedCodes(this, list_no, i).get());
+            }
+
+            n_added += i2 - i1;
+        } else if (subset_type == SUBSET_TYPE_INVLIST) {
+            if (list_no >= a1 && list_no < a2) {
+                oivf.add_entries(
+                        list_no,
+                        n,
+                        ScopedIds(this, list_no).get(),
+                        ScopedCodes(this, list_no).get());
+                n_added += n;
+            }
+        }
+        accu_n += n;
+    }
+    return n_added;
+}
+
 double InvertedLists::imbalance_factor() const {
     std::vector<int> hist(nlist);
 
@@ -109,7 +214,9 @@ void InvertedLists::print_stats() const {
     }
     for (size_t i = 0; i < sizes.size(); i++) {
         if (sizes[i]) {
-            printf("list size in < %d: %d instances\n", 1 << i, sizes[i]);
+            printf("list size in < %zu: %d instances\n",
+                   static_cast<size_t>(1) << i,
+                   sizes[i]);
         }
     }
 }
@@ -158,7 +265,7 @@ const uint8_t* ArrayInvertedLists::get_codes(size_t list_no) const {
     return codes[list_no].data();
 }
 
-const InvertedLists::idx_t* ArrayInvertedLists::get_ids(size_t list_no) const {
+const idx_t* ArrayInvertedLists::get_ids(size_t list_no) const {
     assert(list_no < nlist);
     return ids[list_no].data();
 }
@@ -267,7 +374,7 @@ void HStackInvertedLists::release_codes(size_t, const uint8_t* codes) const {
     delete[] codes;
 }
 
-const Index::idx_t* HStackInvertedLists::get_ids(size_t list_no) const {
+const idx_t* HStackInvertedLists::get_ids(size_t list_no) const {
     idx_t *ids = new idx_t[list_size(list_no)], *c = ids;
 
     for (int i = 0; i < ils.size(); i++) {
@@ -281,8 +388,7 @@ const Index::idx_t* HStackInvertedLists::get_ids(size_t list_no) const {
     return ids;
 }
 
-Index::idx_t HStackInvertedLists::get_single_id(size_t list_no, size_t offset)
-        const {
+idx_t HStackInvertedLists::get_single_id(size_t list_no, size_t offset) const {
     for (int i = 0; i < ils.size(); i++) {
         const InvertedLists* il = ils[i];
         size_t sz = il->list_size(list_no);
@@ -312,8 +418,6 @@ void HStackInvertedLists::prefetch_lists(const idx_t* list_nos, int nlist)
 
 namespace {
 
-using idx_t = InvertedLists::idx_t;
-
 idx_t translate_list_no(const SliceInvertedLists* sil, idx_t list_no) {
     FAISS_THROW_IF_NOT(list_no >= 0 && list_no < sil->nlist);
     return list_no + sil->i0;
@@ -349,12 +453,11 @@ void SliceInvertedLists::release_codes(size_t list_no, const uint8_t* codes)
     return il->release_codes(translate_list_no(this, list_no), codes);
 }
 
-const Index::idx_t* SliceInvertedLists::get_ids(size_t list_no) const {
+const idx_t* SliceInvertedLists::get_ids(size_t list_no) const {
     return il->get_ids(translate_list_no(this, list_no));
 }
 
-Index::idx_t SliceInvertedLists::get_single_id(size_t list_no, size_t offset)
-        const {
+idx_t SliceInvertedLists::get_single_id(size_t list_no, size_t offset) const {
     return il->get_single_id(translate_list_no(this, list_no), offset);
 }
 
@@ -380,8 +483,6 @@ void SliceInvertedLists::prefetch_lists(const idx_t* list_nos, int nlist)
 
 namespace {
 
-using idx_t = InvertedLists::idx_t;
-
 // find the invlist this number belongs to
 int translate_list_no(const VStackInvertedLists* vil, idx_t list_no) {
     FAISS_THROW_IF_NOT(list_no >= 0 && list_no < vil->nlist);
@@ -449,14 +550,13 @@ void VStackInvertedLists::release_codes(size_t list_no, const uint8_t* codes)
     return ils[i]->release_codes(list_no, codes);
 }
 
-const Index::idx_t* VStackInvertedLists::get_ids(size_t list_no) const {
+const idx_t* VStackInvertedLists::get_ids(size_t list_no) const {
     int i = translate_list_no(this, list_no);
     list_no -= cumsz[i];
     return ils[i]->get_ids(list_no);
 }
 
-Index::idx_t VStackInvertedLists::get_single_id(size_t list_no, size_t offset)
-        const {
+idx_t VStackInvertedLists::get_single_id(size_t list_no, size_t offset) const {
     int i = translate_list_no(this, list_no);
     list_no -= cumsz[i];
     return ils[i]->get_single_id(list_no, offset);
