@@ -34,7 +34,7 @@ double eval_codec_error(long ncentroids, long m, const std::vector<float>& v) {
 
     // encode and decode to compute reconstruction error
 
-    std::vector<faiss::Index::idx_t> keys(nb);
+    std::vector<faiss::idx_t> keys(nb);
     std::vector<uint8_t> codes(nb * m);
     index.encode_multiple(nb, keys.data(), v.data(), codes.data(), true);
 
